﻿using GeneralGommunication;
using System;
using System.Collections.Generic;
using System.ComponentModel;

namespace FlyXray
{
    public class FlyXrayCore : IFlyXrayCore 
    { 
        public event PropertyChangedEventHandler PropertyChanged;

        public bool IsError => !string.IsNullOrEmpty(ErrMsg);

        public string ErrMsg { get; private set; }

        public int ErrCnt { get; private set; }

        public bool IsConnected { get; private set; }

        public int ID { get; set; }


        ModbusAsciiAsync modbus;

        public void Init(ModbusAsciiAsync modbus)
        {
            this.modbus = modbus;
            this.modbus.DeviceConnectEvent += Modbus_DeviceConnectEvent;
        }

        private void Modbus_DeviceConnectEvent(object sender, DeviceConnectEventArgs e)
        {
            if (e.deviceNo == ID)
            {
                IsConnected = e.isConnected;
                if (ErrMsg != e.errMsg)
                {
                    ErrMsg = e.errMsg;
                    if (IsError)
                    {
                        ErrCnt++;
                    }
                }
            }
        }

        /// <summary>
        /// 获取参数
        /// </summary>
        [CallBack(typeof(GetParam_Response))]
        public void GetParam(CallBackHandler asyncDelegate, object asyncContext)
        {
            Modbus_Transaction tran = modbus.Do_03(
                ID, 1, 12, nameof(GetParam),
                (values) =>
                {
                    var response = new GetParam_Response();
                    response.HvSet = values.U16ToU32_Little_Endian(0);
                    response.IanSet = values.U16ToU32_Little_Endian(2);
                    response.IlimSet = values.U16ToU32_Little_Endian(4);
                    response.HvMax= values.U16ToU32_Little_Endian(6);
                    response.IanMax = values.U16ToU32_Little_Endian(8);
                    response.IpreSet = values.U16ToU32_Little_Endian(10);

                    return response;
                }, asyncDelegate, asyncContext);


            modbus.AddTran(tran);
        }






        /// <summary>
        /// 设置参数
        /// </summary>
        /// <param name="hvSet">高压设置</param>
        /// <param name="ianSet">管流设置</param>
        /// <param name="ilimSet">灯丝限制设置</param>
        /// <param name="hvMax">高压最大值</param>
        /// <param name="ianMax">管流最大值</param>
        /// <param name="ipreSet">灯丝预热设置</param>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        public void SetParam(
            UInt32 hvSet,
            UInt32 ianSet,
            UInt32 ilimSet,
            UInt32 hvMax,
            UInt32 ianMax,
            UInt32 ipreSet,
            CallBackHandler asyncDelegate, object asyncContext)
        {
            var u16s = new List<UInt16>();
            u16s.AddRange(((UInt32)hvSet).U32ToU16_Little_Endian());
            u16s.AddRange(((UInt32)ianSet).U32ToU16_Little_Endian());
            u16s.AddRange(((UInt32)ilimSet).U32ToU16_Little_Endian());
            u16s.AddRange(((UInt32)hvMax).U32ToU16_Little_Endian());
            u16s.AddRange(((UInt32)ianMax).U32ToU16_Little_Endian());
            u16s.AddRange(((UInt32)ipreSet).U32ToU16_Little_Endian());

            Modbus_Transaction tran = modbus.Do_10(
                ID, 1, u16s, nameof(SetParam),
                asyncDelegate, asyncContext);

            modbus.AddTran(tran);
        }

        /// <summary>
        /// 保存参数
        /// </summary>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        public void SaveParam(CallBackHandler asyncDelegate, object asyncContext)
        {
            Modbus_Transaction tran = modbus.Do_06(
            ID, 0, 0x8000, nameof(SaveParam),
            asyncDelegate, asyncContext);

            modbus.AddTran(tran);
        }

        /// <summary>
        /// 获取当前值
        /// </summary>
        /// <param name="asyncDelegate"></param>
        /// <param name="asyncContext"></param>
        [CallBack(typeof(GetCurrentValue_Response))]
        public void GetCurrentValue(CallBackHandler asyncDelegate, object asyncContext)
        {
            Modbus_Transaction tran;

            tran = modbus.Do_04(
                ID, 1, 4, nameof(GetCurrentValue),
                (values) =>
                {
                    var response = new GetCurrentValue_Response();
                    response.HvCurrent = values.U16ToU32_Little_Endian(0);
                    response.IanCurrent = values.U16ToU32_Little_Endian(2);
                    return response;
                }, asyncDelegate, asyncContext);

            tran.EnableLog = false;
            modbus.AddTran(tran);
        }
    }
}
